<?php
/**
 * 虚拟模型类文件
 * 
 * @author dongbeta <http://dongbeta.com>
 * @copyright Copyright @ 2012 Yaodong Zhao
 * @license http://www.opensource.org/licenses/bsd-license.php
 */

/**
 * 虚拟数据模型，根据配置文件类实现类似CActiveRecord的使用
 * 用于CForm等需要用到Yii模型的类中。配置文件格式如下：
 * return array(
 * 	'site_title'=>array(
 * 		'type'=>'text',
 * 		'maxlength'=>80,
 * 		'label'=>'Open user register',
 *      'default'=>'Yii Site',
 *     	'rules'=>array(
 *         array('site_title', 'length', 'max'=>64),
 *     )
 * 	),
 *  ...
 * );
 * 
 * @version $Id$
 */
abstract class XVirtualModel extends CModel
{
    private $_attributes = array(); //字段，数组，格式为：字段名=>字段配置
    private $_rules = array(); //字段验证规则，数组，格式为：字段名=>验证规则数组
    private $_labels = array(); //字段标注，数组，格式为：字段名=>字段标注
    private $_values = array(); //字段的值，数组，格式为：字段名=>字段值
    
    /**
     * 构造器，通过传入配置文件的地址别名来构造模型。
     * @param mixed $config 配置文件地址别名或配置数组
     */
    public function __construct($config = null)
    {
        if (!is_null($config)) {
            $this->configure($config);
            $this->loadValues();
        }
        $this->init();
    }
    
    /**
     * 初始化，会在构造函数的最后运行
     */
    public function init()
    {
    }
    
    /**
     * 返回模型的全部字段名列表
     * @return array 字段名列表
     */
    public function attributeNames()
    {
        return array_keys($this->_attributes);
    }
    
    /**
     * 返回所有字段的标签
     * @return array 字段的标签数组 
     */
    public function attributeLabels()
    {
    	return $this->_labels;
    }
    
    /**
     * 返回所有字段的验证规则
     * @return array 字段的规则数组
     */
    public function rules()
    {
        $rules = array();
        foreach ($this->_rules as $attributeRules) {
            $rules = array_merge($rules, $attributeRules);
        }
        return $rules;
    }
    
    /**
     * 保存值当前的值，你需要重写此函数来以你的方式来保存
     * @return boolean 是否保存成功
     */
    abstract public function save();
    
    /**
     * 根据传入的参数来配置model，可以重写此函数以实现特定的配置参数
     * @param mixed $config 配置文件别名
     */
    protected function configure($config)
    {
        if (is_string($config)) {
            $config = Yii::getPathOfAlias($config).'.php';
            $config = require($config);
        }
        foreach ($config as $name => $attribute) 
            $this->addAttribute($name, $attribute);
    }
    
    /**
     * 载入全部字段的值
     */
    abstract public function loadValues();
    
    /**
     * 魔术函数，优先设置字段值
     * @param $name 字段名
     * @param $value 字段值
     */
    public function __set($name, $value)
    {
    	if (isset($this->_attributes[$name]))
    	    $this->_values[$name] = $value;
    	else
    	    parent::__set($name, $value);
    }
    
    /**
     * 魔术函数，优先获得字段值
     * @param $name 字段名
     */
    public function __get($name)
    {
        if (isset($this->_attributes[$name]))
            return isset($this->_values[$name])?$this->_values[$name]:'';
    	else 
    	    return parent::__get($name);
    }
    
    /**
     * 增加一个字段
     * @param string $name
     * @param array $attribute
     */
    public function addAttribute($name, $attribute)
    {
    	if (isset($attribute['rules'])) {
    	    $this->_rules[$name] = $attribute['rules'];
    	    unset($attribute['rules']);
    	}else{
    	    $this->_rules[$name] = array(array($name, 'safe'));
    	}
    	if (isset($attribute['label'])) {
    	    $this->_labels[$name] = $attribute['label'];
    	    unset($attribute['label']);
    	}
        if (isset($attribute['default'])) {
    	    $this->_values[$name] = $attribute['default'];
    	    unset($attribute['default']);
    	}
    	$this->_attributes[$name] = $attribute;
    }
    
    /**
     * 移除一个字段
     * @param $name 字段名
     */
    public function removeAttribute($name)
    {
    	unset($this->_labels[$name], $this->_values[$name], $this->_rules[$name]);
    }
    
    /**
     * 设置字段值
     * @param $name 字段名
     * @param $value 字段值
     */
    public function setValue($name, $value)
    {
    	$this->_values[$name] = $value;
    }
    
    /**
     * 取得字段值
     * @param $name 字段名
     * @return mixed 返回此字段的值 
     */
    public function getValue($name)
    {
    	return $this->_values[$name];
    }
    
    /**
     * 获得表单数组，可以直接用于CForm
     * @return array 可以用户CForm的表单配置数组
     */
    public function getFormMap()
    {
        return empty($this->_attributes)?array():array('elements'=>$this->_attributes);
    }
}